from datetime import datetime
from django.dispatch import receiver
from django.utils.timezone import make_aware
from .signals import subscription_created, subscription_cancelled, subscription_updated
from .models import PaddlePlan, PaddleSubscription
from .utils import get_account_by_passthrough


@receiver(subscription_created)
def subscription_created_receiver(**kwargs):
    payload = kwargs['payload']

    account = get_account_by_passthrough(payload['passthrough'])

    try:
        plan = PaddlePlan.objects.get(id=payload['subscription_plan_id'])
    except PaddlePlan.DoesNotExist:
        # raise warning/exception here
        plan = None

    PaddleSubscription.objects.create(
        id=payload['subscription_id'],
        account=account,
        plan=plan,
        user_id=payload['user_id'],
        user_email=payload['email'],
        marketing_consent=bool(payload['marketing_consent']),
        update_url=payload['update_url'],
        cancel_url=payload['cancel_url'],
        state=payload['status'],
        signup_date=make_aware(datetime.strptime(payload['event_time'], '%Y-%m-%d %H:%M:%S')),
        cancellation_effective_date=None
    )


@receiver(subscription_cancelled)
def subscription_cancelled_receiver(**kwargs):
    payload = kwargs['payload']

    try:
        subscription = PaddleSubscription.objects.get(id=payload['subscription_id'])
    except PaddleSubscription.DoesNotExist:
        # raise warning/exception here
        pass

    subscription.cancellation_effective_date = make_aware(datetime.strptime(payload['cancellation_effective_date'], '%Y-%m-%d'))
    subscription.state = payload['status']
    subscription.save()


@receiver(subscription_updated)
def subscription_updated_receiver(**kwargs):
    payload = kwargs['payload']

    try:
        plan = PaddlePlan.objects.get(id=payload['subscription_plan_id'])
    except PaddlePlan.DoesNotExist:
        # raise warning/exception here
        plan = None

    try:
        subscription = PaddleSubscription.objects.get(id=payload['subscription_id'])
    except PaddleSubscription.DoesNotExist:
        # raise warning/exception here
        pass

    subscription.state = payload['status']
    subscription.plan = plan
    subscription.save()



from django.contrib import admin
from django_paddle.models import PaddlePlan, PaddleInitialPrice, PaddleRecurringPrice, PaddleSubscription


class PaddleInitialPriceInline(admin.TabularInline):
    model = PaddleInitialPrice


class PaddleRecurringPriceInline(admin.TabularInline):
    model = PaddleRecurringPrice


class PaddlePlanAdmin(admin.ModelAdmin):
    list_display = ['id', 'name', 'billing_type', 'billing_period']
    inlines = [PaddleInitialPriceInline, PaddleRecurringPriceInline]


class PaddleSubscriptionAdmin(admin.ModelAdmin):
    list_display = ['id', 'signup_date', 'user_email', 'state']


admin.site.register(PaddlePlan, PaddlePlanAdmin)
admin.site.register(PaddleSubscription, PaddleSubscriptionAdmin)



from django.core.management.base import BaseCommand
from django_paddle.models import PaddleSubscription, PaddlePayment


class Command(BaseCommand):
    help = 'Sync payments'

    def handle(self, *args, **options):
        for subscription in PaddleSubscription.objects.all():
            subscription.sync_payments()



from django.http import HttpResponse
from .utils import webhook_signature_is_valid
from django.views.decorators.http import require_POST
from django.views.decorators.csrf import csrf_exempt
from django_paddle import signals


class Webhook:
    pass


@csrf_exempt
@require_POST
def webhook(request):
    payload = request.POST.dict()

    if webhook_signature_is_valid(payload):
        alert_name = payload['alert_name']
        signal = getattr(signals, alert_name)
        if signal:
            signal.send(
                sender=Webhook,
                payload=payload
            )
        else:
            # Log warning if alert_name does not match any signal?
            pass

    return HttpResponse()



# Generated by Django 3.2.4 on 2021-11-28 15:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_paddle', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='paddlesubscription',
            name='cancellation_effective_date',
            field=models.DateTimeField(default=None, null=True, blank=True),
        ),
    ]



from copy import copy
import requests
from django.conf import settings


class PaddleClient:

    def __init__(self):
        self.base_url = 'https://vendors.paddle.com/api/2.0/'
        self.vendor_id = settings.PADDLE_VENDOR_ID
        self.vendor_auth_code = settings.PADDLE_AUTH_CODE
        self.base_payload = {
            'vendor_id': self.vendor_id,
            'vendor_auth_code': self.vendor_auth_code
        }

    # Plans

    def plans_list(self):
        rsp = requests.post(
            url=self.base_url + 'subscription/plans',
            json=self.base_payload
        )
        return rsp.json()['response']

    def plans_get(self, plan_id):
        payload = copy(self.base_payload)
        payload.update(plan_id=plan_id)
        rsp = requests.post(
            url=self.base_url + 'subscription/plans',
            json=payload
        )
        return rsp.json()['response'][0]

    # Subscriptions

    def subscriptions_list(self, state=None):

        """
        :param state:   filter by state, returns all active, past_due, trialing
                        and paused subscription plans if not specified.
                        Will NOT return deleted subscriptions
                See https://developer.paddle.com/api-reference/subscription-api/users/listusers
        """

        subscriptions = []
        max_results = 200
        payload = copy(self.base_payload)
        payload.update(page=1, results_per_page=max_results)

        if state:
            payload.update(state=state)

        while True:
            data = requests.post(
                url=self.base_url + 'subscription/users',
                json=payload
            ).json()['response']
            subscriptions += data
            if len(data) < max_results:
                break
            else:
                payload['page'] += 1

        return subscriptions

    def subscriptions_get(self, subscription_id):
        payload = copy(self.base_payload)
        payload.update(subscription_id=subscription_id)
        return requests.post(
            url=self.base_url + 'subscription/users',
            json=payload
        ).json()['response'][0]

    def subscriptions_cancel(self, subscription_id):
        payload = copy(self.base_payload)
        payload.update(
            subscription_id=subscription_id
        )
        requests.post(
            url=self.base_url + 'subscription/users_cancel',
            json=payload
        )

    def subscriptions_pause(self, subscription_id):
        payload = copy(self.base_payload)
        payload.update(
            subscription_id=subscription_id,
            pause=True
        )
        requests.post(
            url=self.base_url + 'subscription/users/update',
            json=payload
        )

    def subscriptions_unpause(self, subscription_id):
        payload = copy(self.base_payload)
        payload.update(
            subscription_id=subscription_id,
            pause=False
        )
        requests.post(
            url=self.base_url + 'subscription/users/update',
            json=payload
        )

    # Payments

    def payments_list(self, subscription_id=None, is_paid=None):
        payload = copy(self.base_payload)
        if subscription_id:
            payload.update(subscription_id=subscription_id)
        if is_paid is not None:
            payload.update(is_paid=is_paid)
        return requests.post(
            url=self.base_url + 'subscription/payments',
            json=payload
        ).json()['response']

    # Transactions

    def transactions_list(self, entity, id):
        payload = copy(self.base_payload)
        return requests.post(
            url=self.base_url + '{}/{}/transactions'.format(entity, id),
            json=payload
        ).json()['response']
